var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};

// src/core/component/action-row-component.ts
function actionRowComponent(...children) {
  return {
    type: "actionRow",
    children: children.flat()
  };
}

// src/core/component/button-component.ts
import { randomUUID } from "node:crypto";
function buttonComponent(options) {
  return __spreadProps(__spreadValues({}, options), {
    type: "button",
    customId: randomUUID()
  });
}

// src/core/component/embed-component.ts
function embedComponent(embed) {
  return { type: "embed", embed };
}

// src/core/component/link-component.ts
function linkComponent(options) {
  return __spreadProps(__spreadValues({}, options), {
    type: "link"
  });
}

// src/core/component/select-menu-component.ts
import { randomUUID as randomUUID2 } from "node:crypto";
function selectMenuComponent(_a) {
  var _b = _a, {
    options,
    selected
  } = _b, args = __objRest(_b, [
    "options",
    "selected"
  ]);
  const selectedOptions = typeof selected === "string" ? new Set([selected]) : selected != null ? new Set(selected) : new Set();
  return __spreadProps(__spreadValues({}, args), {
    type: "selectMenu",
    customId: randomUUID2(),
    options: options.map((option) => __spreadProps(__spreadValues({}, option), {
      default: option.default ?? selectedOptions.has(option.value)
    }))
  });
}

// src/core/gatekeeper.ts
import chalk3 from "chalk";
import glob from "fast-glob";
import { relative } from "node:path";

// src/internal/helpers.ts
function raise(error) {
  throw typeof error === "string" ? new Error(error) : error;
}
function toError(value) {
  return value instanceof Error ? value : new Error(String(value));
}
function isTruthy(value) {
  return !!value;
}
function isNonNil(value) {
  return value != null;
}
function last(array) {
  return array[array.length - 1];
}
function isDeepEqual(a, b) {
  if (Object.is(a, b))
    return true;
  if (a == null || b == null)
    return false;
  if (typeof a !== "object" || typeof b !== "object")
    return false;
  if (Object.keys(a).length !== Object.keys(b).length)
    return false;
  for (const key in a) {
    if (!isDeepEqual(a[key], b[key]))
      return false;
  }
  return true;
}

// src/internal/load-file.ts
import { createRequire } from "node:module";
var require2;
async function loadFile(path) {
  try {
    return await import(path);
  } catch (error) {
    try {
      require2 ??= createRequire(import.meta.url);
      return require2(path);
    } catch {
      throw error;
    }
  }
}

// src/internal/logger.ts
import chalk from "chalk";
function createConsoleLogger(config = {}) {
  const prefix = config.name ? chalk.gray`[${config.name}]` : "";
  const levels = new Set(config.levels || ["info", "success", "error", "warn"]);
  const logger = {
    info(...args) {
      if (levels.has("info")) {
        console.info(prefix, chalk.cyan`[i]`, ...args);
      }
    },
    success(...args) {
      if (levels.has("success")) {
        console.info(prefix, chalk.green`[s]`, ...args);
      }
    },
    error(...args) {
      if (levels.has("error")) {
        console.error(prefix, chalk.red`[e]`, ...args);
      }
    },
    warn(...args) {
      if (levels.has("warn")) {
        console.warn(prefix, chalk.yellow`[w]`, ...args);
      }
    },
    async promise(description, promise) {
      const startTime = Date.now();
      try {
        logger.info(description, chalk.gray`...`);
        const result = await promise;
        logger.success(description, chalk.green`done`, chalk.gray`(${Date.now() - startTime}ms)`);
        return result;
      } catch (error) {
        logger.error(description, chalk.red`failed`, chalk.gray`(${Date.now() - startTime}ms)`);
        logger.error(toError(error).stack || toError(error).message);
        throw error;
      }
    },
    async block(description, block) {
      return logger.promise(description, block());
    }
  };
  return logger;
}
function createNoopLogger() {
  return {
    info() {
    },
    success() {
    },
    error() {
    },
    warn() {
    },
    async promise(_description, promise) {
      return promise;
    },
    async block(_description, block) {
      return block();
    }
  };
}

// src/core/command/command.ts
import chalk2 from "chalk";
import { randomUUID as randomUUID3 } from "node:crypto";

// src/internal/action-queue.ts
var ActionQueue = class {
  constructor(config) {
    this.actions = [];
    this.running = false;
    this.config = config;
  }
  addAction(action) {
    this.actions.push(action);
    this.actions.sort((a, b) => (a.priority ?? Infinity) - (b.priority ?? Infinity));
    this.runActions();
  }
  runActions() {
    if (this.running)
      return;
    this.running = true;
    queueMicrotask(async () => {
      let action;
      while (action = this.actions.shift()) {
        try {
          await action.run();
        } catch (error) {
          this.config.onError(action.name, error);
        }
      }
      this.running = false;
    });
  }
};

// src/core/component/reply-component.ts
function collectReplyComponents(items) {
  const components = [];
  for (const child of items) {
    if (typeof child === "boolean" || child == null)
      continue;
    if (typeof child === "string" || typeof child === "number") {
      components.push({ type: "text", text: String(child) });
      continue;
    }
    if (Array.isArray(child)) {
      components.push(...collectReplyComponents(child));
      continue;
    }
    components.push(child);
  }
  return components;
}
function flattenRenderResult(result) {
  const ungroupedComponents = collectReplyComponents([result].flat());
  const components = [];
  for (const component of ungroupedComponents) {
    const lastComponent = last(components);
    if (component.type === "button" || component.type === "link") {
      if ((lastComponent == null ? void 0 : lastComponent.type) === "actionRow" && lastComponent.children.every((child) => child.type !== "selectMenu") && lastComponent.children.length < 5) {
        lastComponent.children.push(component);
        continue;
      }
      components.push({
        type: "actionRow",
        children: [component]
      });
      continue;
    }
    if (component.type === "selectMenu") {
      if ((lastComponent == null ? void 0 : lastComponent.type) === "actionRow" && lastComponent.children.length === 0) {
        lastComponent.children.push(component);
        continue;
      }
      components.push({
        type: "actionRow",
        children: [component]
      });
      continue;
    }
    components.push(component);
  }
  return components.filter((component) => {
    const isEmptyActionRow = component.type === "actionRow" && component.children.length === 0;
    return !isEmptyActionRow;
  });
}
function createInteractionReplyOptions(components) {
  var _a, _b;
  const content = components.map((component) => component.type === "text" ? component.text : void 0).filter(isNonNil).join("\n");
  const embeds = components.map((component) => component.type === "embed" && component.embed).filter(isTruthy);
  const replyComponents = components.map((component) => {
    if (component.type !== "actionRow")
      return;
    return {
      type: "ACTION_ROW",
      components: component.children.map((child) => {
        if (child.type === "selectMenu") {
          return __spreadProps(__spreadValues({}, child), { type: "SELECT_MENU" });
        }
        if (child.type === "link") {
          return __spreadProps(__spreadValues({}, child), { style: "LINK", type: "BUTTON" });
        }
        return __spreadProps(__spreadValues({}, child), { type: "BUTTON" });
      })
    };
  });
  const options = {
    content,
    embeds,
    components: replyComponents.filter(isTruthy)
  };
  if (options.content === "") {
    delete options.content;
  }
  const hasComponents = (_a = options.components) == null ? void 0 : _a.length;
  const hasContent = options.content || ((_b = options.embeds) == null ? void 0 : _b.length);
  if (hasComponents && !hasContent) {
    options.content = "_ _";
  }
  return options;
}

// src/core/interaction-context.ts
function createInteractionContext({
  interaction,
  command
}) {
  return {
    user: interaction.user,
    channel: interaction.channel ?? void 0,
    guild: interaction.guild ?? void 0,
    member: interaction.member ?? void 0,
    reply: (render) => {
      const id = command.createReply(render, interaction);
      return {
        get message() {
          return command.getReplyMessage(id);
        },
        refresh: () => {
          command.refreshReply(id);
        },
        delete: () => {
          command.deleteReply(id);
        }
      };
    },
    ephemeralReply: (render) => {
      command.createEphemeralReply(render, interaction);
    },
    defer: () => {
      command.defer(interaction);
    },
    ephemeralDefer: () => {
      command.ephemeralDefer(interaction);
    }
  };
}

// src/core/reply-instance.ts
var PublicReplyInstance = class {
  constructor(render, events) {
    this.renderResult = [];
    this.isDeleted = false;
    this.render = render;
    this.events = events;
  }
  getMessage() {
    return this.message;
  }
  async createMessage(interaction) {
    var _a;
    this.renderResult = flattenRenderResult(this.render());
    if (this.renderResult.length === 0) {
      await this.deleteMessage();
      return;
    }
    const options = createInteractionReplyOptions(this.renderResult);
    if (interaction.deferred && interaction.ephemeral) {
      this.message = await ((_a = interaction.channel) == null ? void 0 : _a.send(options));
      return;
    }
    if (interaction.deferred) {
      this.message = await interaction.editReply(options);
      return;
    }
    if (interaction.replied) {
      this.message = await interaction.followUp(options);
      return;
    }
    this.message = await interaction.reply(__spreadProps(__spreadValues({}, options), {
      fetchReply: true
    }));
  }
  async deleteMessage() {
    const message = this.message;
    this.message = void 0;
    this.isDeleted = true;
    this.events.onDelete(this);
    await (message == null ? void 0 : message.delete());
  }
  async refreshMessage() {
    var _a;
    if (this.isDeleted)
      return;
    this.renderResult = flattenRenderResult(this.render());
    if (this.renderResult.length === 0) {
      await this.deleteMessage();
      return;
    }
    await ((_a = this.message) == null ? void 0 : _a.edit(createInteractionReplyOptions(this.renderResult)));
  }
  findInteractionSubject(interaction) {
    return getInteractiveComponents(this.renderResult).find((it) => it.customId === interaction.customId);
  }
  async updateMessageFromComponentInteraction(interaction) {
    if (this.isDeleted)
      return;
    this.renderResult = flattenRenderResult(this.render());
    if (this.renderResult.length === 0) {
      await this.deleteMessage();
      return;
    }
    if (interaction.deferred || interaction.replied) {
      await this.refreshMessage();
      return;
    }
    await interaction.update(createInteractionReplyOptions(this.renderResult));
  }
};
var EphemeralReplyInstance = class {
  constructor(render) {
    this.renderResult = [];
    this.render = render;
  }
  getMessage() {
    return void 0;
  }
  async createMessage(interaction) {
    this.renderResult = flattenRenderResult(this.render());
    if (this.renderResult.length === 0)
      return;
    const options = createInteractionReplyOptions(this.renderResult);
    if (interaction.replied || interaction.deferred) {
      await interaction.followUp(__spreadProps(__spreadValues({}, options), { ephemeral: true }));
      return;
    }
    await interaction.reply(__spreadProps(__spreadValues({}, options), { ephemeral: true }));
  }
  async deleteMessage() {
  }
  async refreshMessage() {
  }
  findInteractionSubject(interaction) {
    return getInteractiveComponents(this.renderResult).find((it) => it.customId === interaction.customId);
  }
  async updateMessageFromComponentInteraction(interaction) {
    this.renderResult = flattenRenderResult(this.render());
    if (this.renderResult.length === 0)
      return;
    const options = createInteractionReplyOptions(this.renderResult);
    if (interaction.deferred) {
      await interaction.followUp(options);
      return;
    }
    await interaction.update(options);
  }
};
async function callInteractionSubject(interaction, subject, command) {
  const message = interaction.message;
  if (interaction.isButton() && subject.type === "button") {
    await subject.onClick(__spreadProps(__spreadValues({}, createInteractionContext({ interaction, command })), {
      message
    }));
  }
  if (interaction.isSelectMenu() && subject.type === "selectMenu") {
    await subject.onSelect(__spreadProps(__spreadValues({}, createInteractionContext({ interaction, command })), {
      message,
      values: interaction.values
    }));
  }
}
function getInteractiveComponents(result) {
  return flattenRenderResult(result).flatMap((component) => {
    return component.type === "actionRow" ? component.children : [];
  }).filter((component) => {
    return component.type === "button" || component.type === "selectMenu";
  });
}

// src/core/command/command.ts
var commandSymbol = Symbol("command");
function createCommand(config) {
  return __spreadProps(__spreadValues({}, config), { [commandSymbol]: true });
}
var deferPriority = 0;
var updatePriority = 1;
var CommandInstance = class {
  constructor(command, logger) {
    this.replyInstances = new Map();
    this.queue = new ActionQueue({
      onError: (actionName, error) => {
        this.logger.error(`An error occurred running action`, chalk2.bold(actionName), `in command`, chalk2.bold(this.command.name));
        this.logger.error(error);
      }
    });
    this.command = command;
    this.logger = logger;
  }
  createReply(render, interaction) {
    const id = randomUUID3();
    const instance = new PublicReplyInstance(render, {
      onDelete: () => this.replyInstances.delete(id)
    });
    this.replyInstances.set(id, instance);
    this.queue.addAction({
      name: "reply",
      run: () => instance.createMessage(interaction)
    });
    return id;
  }
  getReplyMessage(id) {
    var _a;
    return (_a = this.replyInstances.get(id)) == null ? void 0 : _a.getMessage();
  }
  refreshReply(id) {
    this.queue.addAction({
      name: "refresh",
      run: async () => {
        var _a;
        return (_a = this.replyInstances.get(id)) == null ? void 0 : _a.refreshMessage();
      }
    });
  }
  deleteReply(id) {
    this.queue.addAction({
      name: "replyInstance.deleteMessage",
      run: async () => {
        var _a;
        return (_a = this.replyInstances.get(id)) == null ? void 0 : _a.deleteMessage();
      }
    });
  }
  createEphemeralReply(render, interaction) {
    const id = randomUUID3();
    const instance = new EphemeralReplyInstance(render);
    this.replyInstances.set(id, instance);
    this.queue.addAction({
      name: "replyInstance.createEphemeralMessage",
      run: () => instance.createMessage(interaction)
    });
    return id;
  }
  async handleComponentInteraction(interaction) {
    for (const [, instance] of this.replyInstances) {
      const subject = instance.findInteractionSubject(interaction);
      if (subject) {
        await callInteractionSubject(interaction, subject, this);
        this.queue.addAction({
          name: "replyInstance.handleComponentInteraction",
          priority: updatePriority,
          run: () => instance.updateMessageFromComponentInteraction(interaction, subject, this)
        });
        return true;
      }
    }
    return false;
  }
  defer(interaction) {
    this.queue.addAction({
      name: "defer",
      priority: deferPriority,
      run: async () => {
        if (interaction.deferred)
          return;
        if (interaction.isMessageComponent())
          return interaction.deferUpdate();
        return interaction.deferReply();
      }
    });
  }
  ephemeralDefer(interaction) {
    this.queue.addAction({
      name: "ephemeralDefer",
      priority: deferPriority,
      run: async () => {
        if (interaction.deferred)
          return;
        if (interaction.isMessageComponent())
          return interaction.deferUpdate();
        return interaction.deferReply({ ephemeral: true });
      }
    });
  }
};

// src/core/command/message-command.ts
function createMessageCommands(config) {
  const names = [config.name, ...config.aliases || []];
  return names.map((name) => createCommand({
    name,
    matchesExisting: (appCommand) => {
      return appCommand.name === name && appCommand.type === "MESSAGE";
    },
    register: async (commandManager) => {
      await commandManager.create({
        type: "MESSAGE",
        name
      });
    },
    matchesInteraction: (interaction) => interaction.isContextMenu() && interaction.targetType === "MESSAGE" && interaction.commandName === name,
    run: async (interaction, command) => {
      const isMessageInteraction = interaction.isContextMenu() && interaction.channel && interaction.targetType === "MESSAGE";
      if (!isMessageInteraction)
        raise("Expected a context menu message interaction");
      const targetMessage = await interaction.channel.messages.fetch(interaction.targetId);
      await config.run(__spreadProps(__spreadValues({}, createInteractionContext({ interaction, command })), {
        targetMessage
      }));
    }
  }));
}

// src/core/command/slash-command.ts
import { GuildMember, User } from "discord.js";
function sortChannelTypes(arrA) {
  const channelTypesOrder = [
    "GUILD_TEXT",
    "DM",
    "GUILD_VOICE",
    "GROUP_DM",
    "GUILD_CATEGORY",
    "GUILD_NEWS",
    "GUILD_STORE",
    "GUILD_NEWS_THREAD",
    "GUILD_PUBLIC_THREAD",
    "GUILD_PRIVATE_THREAD",
    "GUILD_STAGE_VOICE"
  ];
  return [...arrA].sort((a, b) => channelTypesOrder.indexOf(a) - channelTypesOrder.indexOf(b));
}
function createSlashCommands(config) {
  const names = [config.name, ...config.aliases || []];
  return names.map((name) => {
    const options = Object.entries(config.options ?? {}).map(([name2, option]) => ({
      name: name2,
      description: option.description,
      type: option.type,
      required: option.required ?? false,
      choices: "choices" in option && option.choices || [],
      channelTypes: "channelTypes" in option && sortChannelTypes(option.channelTypes ?? []) || void 0
    }));
    const commandData = {
      name,
      description: config.description,
      options
    };
    return createCommand({
      name,
      matchesExisting: (command) => {
        if (command.type !== "CHAT_INPUT")
          return false;
        const existingCommandData = {
          name: command.name,
          description: command.description,
          options: command.options.map((option) => ({
            name: option.name,
            description: option.description,
            type: option.type,
            required: option.required,
            choices: "choices" in option && option.choices || [],
            channelTypes: "channelTypes" in option && option.channelTypes || void 0
          }))
        };
        return isDeepEqual(commandData, existingCommandData);
      },
      register: async (manager) => {
        await manager.create(commandData);
      },
      matchesInteraction: (interaction) => {
        return interaction.isCommand() && interaction.commandName === name;
      },
      run: async (interaction, command) => {
        await config.run(__spreadProps(__spreadValues({}, createInteractionContext({ interaction, command })), {
          options: collectSlashCommandOptionValues(config, interaction)
        }));
      }
    });
  });
}
function collectSlashCommandOptionValues(slashCommand, interaction) {
  const options = {};
  for (const [name, optionDefinition] of Object.entries(slashCommand.options ?? {})) {
    if (optionDefinition.type === "STRING") {
      options[name] = interaction.options.getString(name, optionDefinition.required) ?? optionFallback(name, optionDefinition, slashCommand);
    }
    if (optionDefinition.type === "NUMBER") {
      options[name] = interaction.options.getNumber(name, optionDefinition.required) ?? optionFallback(name, optionDefinition, slashCommand);
    }
    if (optionDefinition.type === "INTEGER") {
      options[name] = interaction.options.getInteger(name, optionDefinition.required) ?? optionFallback(name, optionDefinition, slashCommand);
    }
    if (optionDefinition.type === "BOOLEAN") {
      options[name] = interaction.options.getBoolean(name, optionDefinition.required) ?? optionFallback(name, optionDefinition, slashCommand);
    }
    if (optionDefinition.type === "USER") {
      options[name] = interaction.options.getUser(name, optionDefinition.required) ?? optionFallback(name, optionDefinition, slashCommand);
    }
    if (optionDefinition.type === "CHANNEL") {
      const channel = interaction.options.getChannel(name, optionDefinition.required);
      options[name] = channel ?? optionFallback(name, optionDefinition, slashCommand);
    }
    if (optionDefinition.type === "ROLE") {
      const role = interaction.options.getRole(name, optionDefinition.required);
      options[name] = role ?? optionFallback(name, optionDefinition, slashCommand);
    }
    if (optionDefinition.type === "MENTIONABLE") {
      const value = interaction.options.getMentionable(name, optionDefinition.required);
      options[name] = value ? createResolvedMentionable(value) : optionFallback(name, optionDefinition, slashCommand);
    }
  }
  return options;
}
function createResolvedMentionable(value) {
  if (value instanceof User) {
    return {
      isUser: true,
      user: value,
      guildMember: void 0,
      mention: `<@!${value.id}>`
    };
  }
  if (value instanceof GuildMember) {
    return {
      isUser: true,
      user: value.user,
      guildMember: value,
      mention: `<@!${value.id}>`
    };
  }
  return {
    isUser: false,
    role: value,
    mention: `<@&${value.id}>`
  };
}
function optionFallback(optionName, optionDefinition, slashCommand) {
  return optionDefinition.required ? raise(`Could not get required option "${optionName}" for command "${slashCommand.name}"`) : void 0;
}

// src/core/command/user-command.ts
function createUserCommands(config) {
  const names = [config.name, ...config.aliases || []];
  return names.map((name) => createCommand({
    name,
    matchesExisting: (appCommand) => {
      return appCommand.name === name && appCommand.type === "USER";
    },
    register: async (commandManager) => {
      await commandManager.create({
        name,
        type: "USER"
      });
    },
    matchesInteraction: (interaction) => interaction.isContextMenu() && interaction.targetType === "USER" && interaction.commandName === name,
    run: async (interaction, command) => {
      var _a;
      const isUserInteraction = interaction.isContextMenu() && interaction.targetType === "USER";
      if (!isUserInteraction)
        raise("Expected a context menu user interaction");
      const targetUser = await interaction.client.users.fetch(interaction.targetId);
      const targetGuildMember = await ((_a = interaction.guild) == null ? void 0 : _a.members.fetch({
        user: targetUser
      }));
      await config.run(__spreadProps(__spreadValues({}, createInteractionContext({ interaction, command })), {
        targetUser,
        targetGuildMember
      }));
    }
  }));
}

// src/core/gatekeeper.ts
var Gatekeeper = class {
  constructor(config, logger) {
    this.config = config;
    this.logger = logger;
    this.commands = new Set();
    this.commandInstances = new Set();
  }
  static async create(config) {
    const logger = createGatekeeperLogger(config);
    const instance = new Gatekeeper(config, logger);
    if (config.commandFolder) {
      await instance.loadCommandsFromFolder(config.commandFolder);
    }
    instance.addEventListeners(config.client, config.scope ?? "guild");
    return instance;
  }
  getCommands() {
    return [...this.commands];
  }
  addSlashCommand(config) {
    this.addCommands(createSlashCommands(config));
  }
  addUserCommand(config) {
    this.addCommands(createUserCommands(config));
  }
  addMessageCommand(config) {
    this.addCommands(createMessageCommands(config));
  }
  addCommands(commands) {
    for (const command of commands) {
      this.commands.add(command);
    }
  }
  addEventListeners(client, scope) {
    client.on("ready", this.withErrorHandler(async () => {
      const commandList = [...this.commands].map((command) => chalk3.bold(command.name)).join(", ");
      this.logger.success(`Using commands: ${commandList}`);
      for (const guild of client.guilds.cache.values()) {
        if (scope === "guild" || scope === "both") {
          await this.syncGuildCommands(guild);
        } else {
          await this.removeAllCommands(await guild.commands.fetch(), `in ${guild.name}`);
        }
      }
      if (scope === "global" || scope === "both") {
        await this.syncGlobalCommands(client);
      } else if (client.application) {
        await this.removeAllCommands(await client.application.commands.fetch(), "globally");
      }
    }));
    client.on("guildCreate", this.withErrorHandler(async (guild) => {
      if (scope === "guild" || scope === "both") {
        await this.syncGuildCommands(guild);
      } else {
        await this.removeAllCommands(await guild.commands.fetch(), `in ${guild.name}`);
      }
    }));
    client.on("interactionCreate", this.withErrorHandler(async (interaction) => {
      if (interaction.isCommand() || interaction.isContextMenu()) {
        await this.handleCommandInteraction(interaction);
      }
      if (interaction.isMessageComponent()) {
        await this.handleComponentInteraction(interaction);
      }
    }));
  }
  async loadCommandsFromFolder(folderPath) {
    const localPath = relative(process.cwd(), folderPath).replace(/\\/g, "/");
    await this.logger.block(`Loading commands from ${localPath}`, async () => {
      const files = await glob(`./**/*.{ts,tsx,js,jsx,mjs,cjs,mts,cts}`, {
        cwd: folderPath,
        absolute: true
      });
      await Promise.all(files.map(async (path) => {
        const mod = await loadFile(path);
        const fn = mod.default || mod;
        if (typeof fn === "function")
          fn(this);
      }));
    });
  }
  async syncGuildCommands(guild) {
    await this.logger.block(`Syncing commands for ${guild.name}`, async () => {
      await this.syncCommands(`in ${guild.name}`, guild.commands, await guild.commands.fetch());
    });
  }
  async syncGlobalCommands(client) {
    await this.logger.block(`Syncing global commands`, async () => {
      var _a;
      const commandManager = ((_a = client.application) == null ? void 0 : _a.commands) ?? raise("No client application found");
      await this.syncCommands(`globally`, commandManager, await commandManager.fetch());
    });
  }
  async syncCommands(scope, commandManager, remoteCommands) {
    await this.removeUnusedRemoteCommand(remoteCommands);
    await this.updateChangedCommands(scope, commandManager, remoteCommands);
  }
  async removeUnusedRemoteCommand(remoteCommands) {
    const localCommandNames = new Set([...this.commands].map((c) => c.name));
    const unusedRemoteCommands = remoteCommands.filter((command) => !localCommandNames.has(command.name));
    for (const [, command] of unusedRemoteCommands) {
      await command.delete();
    }
  }
  async updateChangedCommands(scope, commandManager, remoteCommands) {
    for (const command of this.commands) {
      const isExisting = remoteCommands.some((appCommand) => {
        return command.matchesExisting(appCommand);
      });
      if (!isExisting) {
        await command.register(commandManager);
        this.logger.info(`Created ${scope}: ${chalk3.bold(command.name)}`);
      }
    }
  }
  async removeAllCommands(commands, scope) {
    if (commands.size === 0)
      return;
    await this.logger.block(`Removing all commands ${scope}`, async () => {
      for (const [, command] of commands) {
        await command.delete();
      }
    });
  }
  async handleCommandInteraction(interaction) {
    var _a, _b;
    const command = [...this.commands.values()].find((command2) => command2.matchesInteraction(interaction));
    if (!command)
      return;
    const instance = new CommandInstance(command, this.logger);
    this.commandInstances.add(instance);
    try {
      await command.run(interaction, instance);
    } catch (error) {
      this.logger.error(`Error running command`, chalk3.bold(command.name));
      this.logger.error(error);
      (_b = (_a = this.config).onError) == null ? void 0 : _b.call(_a, toError(error));
    }
  }
  async handleComponentInteraction(interaction) {
    for (const context of this.commandInstances) {
      if (await context.handleComponentInteraction(interaction))
        return;
    }
  }
  withErrorHandler(fn) {
    return async (...args) => {
      var _a, _b;
      try {
        return await fn(...args);
      } catch (error) {
        this.logger.error("An error occurred:", error);
        (_b = (_a = this.config).onError) == null ? void 0 : _b.call(_a, toError(error));
      }
    };
  }
};
function createGatekeeperLogger(config) {
  if (config.logging === true || config.logging == null) {
    return createConsoleLogger({ name: config.name });
  }
  if (!config.logging) {
    return createNoopLogger();
  }
  return createConsoleLogger({ name: config.name, levels: config.logging });
}
export {
  Gatekeeper,
  actionRowComponent,
  buttonComponent,
  embedComponent,
  linkComponent,
  selectMenuComponent
};
//# sourceMappingURL=main.js.map
